
; "Midi Digi Drums" By Leif Bloomquist   Schema/AIC  leif@schemafactor.com
;
; Entry into the TinySID2 Compo 2006  (512 byte category)

;"Vars"

;$fb - placeholder for jiffy value when done current note
;$fc - holds location in track

;$fd - always 0
;$fe - points to current instrument - either $09 (kick) or $0a (hihat)

; ================ Setup ======================
  processor 6502 ;C64

  org $0801,00
  
basic  ;"SYS2061"
	dc.b $0c,$08,$64,$0c,$9e,$32,$30
	dc.b $36,$31,$00,$00,$00	

  ;Digibooster  
  lda #$ff
  sta $d406
  sta $d406+7
  sta $d406+14
  lda #$49
  sta $d404
  sta $d404+7
  sta $d404+14
  
  ;Fill in the "513th" byte - last byte of sample 
  lda #$14
  sta $09FF
  
  ldx #$00
  stx $fd
  stx $d011  ; Blank Screen
   
  ; Split up track 
split
  lda sampleraw,x       ; Grab the packed byte
  lsr sampleraw,x       ; Rotate original right
  lsr sampleraw,x       ; Rotate original right
  lsr sampleraw,x       ; Rotate original right
  lsr sampleraw,x       ; Rotate original right - high nybble should be low nybble now
  and #$0f              ; Strip high bits out of accumulator
  sta sampleraw+$100,x  ; Stash original low nybble in next page
  inx
  bne split

; Reset to beginning of track
init 
  ldx #$00 
  stx $fc

; ================ Main Loop ======================

repeat
  ; 1. Get a byte from the "track" and determine note duration.
  ldy $fc
  lda track,y
  
  beq init ; A zero byte means the end of the track.  Start from the beginning again.
  
  ; Mask out the duration bits
  and #$70
  
  ;Divide by 4 and store in the code ahead
  lsr
  lsr
  sta duration+1
  
  ; Figure out the jiffy count at the end of this duration
  lda $a2  ; Current clock time in jiffies
  
duration
  adc #16  ; Ignore carry, we want wraparound  (#16 gets overwritten)
  sta $fb
  
  ; 2. Select instrument.
  ; Default to kick
  lda #>sampleraw
  sta $fe
  
  ; Get the original byte again  
  lda track,y
  
  ; Determine instrument through hi bit
  bmi go    ; Leave as kick  
  inc $fe   ; Move to next page where hi-hat samples are

  ; 3. Determine the playback speed
go
  and #$0f  ; Mask the playback speed
  sta samplespeed+1   ; Store in code ahead
  
  ; 4. Play the sample.
  jsr play

  ; 5. Wait for timeout to expire
  ldy $fb
rest
  cpy $a2
  bne rest

  ; Move to next entry in track  
  inc $fc 
  jmp repeat

; ================ Play Instrument ======================
play
  ldy #$00  
playloop
   lda ($fd),y
   sta $d418
	 sta $d020

samplespeed
	 ldx #12 ;some delay value -  set from track (#12 gets overwritten)
playdelay	 
   dex
	 bne playdelay

	 iny
	 bne playloop  ; Samples are exactly 255 bytes long, so we're done when back to 0
 
kickdone
   rts

; ================ Track ======================
track  ;(Copy/paste from Excel)

   dc.b	76
   dc.b	76
   dc.b	207
   dc.b	44
   dc.b	44
   dc.b	76
   dc.b	76
   dc.b	207
   dc.b	76
   dc.b	33
   dc.b	33
   dc.b	33
   dc.b	33
   dc.b	37
   dc.b	37
   dc.b	37
   dc.b	37
   dc.b	76
   dc.b	76
   dc.b	207
   dc.b	44
   dc.b	44
   dc.b	76
   dc.b	76
   dc.b	207
   dc.b	76
   dc.b	207
   dc.b	76
   dc.b	207
   dc.b	76
   dc.b	175
   dc.b	164
   dc.b	44
   dc.b	164
   dc.b	175
   dc.b	164
   dc.b	44
   dc.b	164
   dc.b	207
   dc.b	76
   dc.b	207
   dc.b	76
   dc.b	175
   dc.b	164
   dc.b	44
   dc.b	164
   dc.b	175
   dc.b	164
   dc.b	44
   dc.b	164
   dc.b	207
   dc.b	76
   dc.b	207
   dc.b	76
   dc.b	175
   dc.b	164
   dc.b	44
   dc.b	164
   dc.b	175
   dc.b	164
   dc.b	44
   dc.b	164
   dc.b	207
   dc.b	76
   dc.b	207
   dc.b	76
   dc.b	175
   dc.b	164
   dc.b	44
   dc.b	164
   dc.b	175
   dc.b	164
   dc.b	44
   dc.b	164
   dc.b	76
   dc.b	76
   dc.b	207
   dc.b	44
   dc.b	44
   dc.b	76
   dc.b	76
   dc.b	207
   dc.b	76
   dc.b	207
   dc.b	76
   dc.b	207
   dc.b	76
   dc.b	175
   dc.b	164
   dc.b	44
   dc.b	164
   dc.b	175
   dc.b	164
   dc.b	44
   dc.b	164
   dc.b	196
   dc.b	196
   dc.b	148
   dc.b	148
   dc.b	148
   dc.b	148
   dc.b	44
   dc.b	44
   dc.b	207
   dc.b	207
   dc.b	175
   dc.b	175
   dc.b	175
   dc.b	175
  
  dc.b 0  ; EOT
 
; ================ Packed Sample ======================
  org $0900 
sampleraw
  incbin "samples-packed.bin"
