; Miniature Rhapsody
; By Eric Odland
;
; Written for "Tiny SID 2" - Online SID Music-Programming Competition
; Platform:Commodore 64
;
; (c) 2006 Stone Monkey Software.
;
; File: mr.S
; Created on 3/22/06.
;
; NOTES:
;	Idea to use undocumented opcode SAX borrowed from Stefano Tognon's xl5


	processor 6502


; ************ VARIABLES ***********
	SEG.U Data
	; Zero Page variables.
	org $E0
voice1Note			dc.b 0
	org $E7
voice2Note			dc.b 0
	org $EE
voice3Note			dc.b 0

	org $FA
pattern1Position	dc.b 0
arpeggioIndex		dc.b 0
arpeggioVector		dc.b 0
	; arpeggio data
	org $C000
arpeggioData	ds.b 17,0



	SEG header
	org $0326  

	dc.b $2A, $03
	dc.b $ED, $F6

; ****************    CODE   *****************	
	SEG code
	org $032A
	
	sei

SetupFrequencyTable:	
	; Setup frequency table
	ldy #11
	lda #1
.1	sta freq_hi,y
	dey
	bpl .1
	ldy #0
.2	lda freq_lo,y
	asl
	sta freq_lo+12,y
	lda freq_hi,y
	rol
	sta freq_hi+12,y
	iny
	bne .2
	tya
	

Start:
	sta pattern1Position
	; Initialize SID
	lda #$9D	; voice1 sustain/release
	sta $D406
	lsr			; voice2 sustain/release
	sta $D40D
	sta $D413	; voice3 attack/decay
	
	; Init player
	bne GetNextArpeggio

UpdateFrame		subroutine
	; update voice 1
	ldx arpeggioIndex
	lda arpeggioData,x
	sta voice1Note
	
	ldx #14
.nextVoice:
	lda voice1Note,x
	tay
	lda freq_lo,y
	sta $D400,x
	lda freq_hi,y
	sta $D401,x
	lda #$21
	sta $D404,x
	lda #$FF
	dc.b $CB, 7			; SAX #7 : X = (A & X)-7 
	bpl .nextVoice
	
	lda arpeggioVector	; arpeggioVector = -arpeggioVector
	ldx arpeggioIndex
	cpx #16
	bne .1
	eor #%11111110
	sta arpeggioVector
.1	clc
	adc arpeggioIndex
	sta arpeggioIndex
	
	bne endGetNextArpeggio
		inc pattern1Position
		
;--------- GETNEXTARPEGGIO ---------
GetNextArpeggio	subroutine
		ldy pattern1Position
		lda PatternTable,y
		beq Start
	; fill arpeggioData
	pha
	and #$0F
	clc
	adc #31
	sta arpeggioData	; store first note in arpeggio
	pla
	
	lsr
	lsr
	lsr
	lsr
	
	pha		; arpeggio/bass note values
	and #%00000011	;extract bass note
	tay
	lda BassNote,y
	sta voice3Note
	
	pla		; extract arpeggio value
	lsr
	lsr
	
	tay
	lda ArpeggioTable,y
	
	ldy #0
	ldx #3

.1	rol
	rol
	pha
	rol
	and #%00000011
	sec
	adc arpeggioData,y
	iny
	sta arpeggioData,y
	
	; setup filter
	; Put this here instead of at initizalization to save space
	lda SIDsnapshot-1,x
	sta $D416-1,x
	
	pla
	dex
	bne .1
	
.2	iny
	lda arpeggioData-4,y
	clc
	adc #12
	sta arpeggioData,y
	cpy #16
	bne .2
	
	stx arpeggioIndex
	; arpeggioVector = 1
	inx
	stx arpeggioVector
	
	lda #$20
	sta $D412
;------- END GETNEXTARPEGGIO ----------
endGetNextArpeggio:	
	ldx #70
.wait	lda $D012
	bne .wait
	dex
	bne .wait
	
	lda voice1Note
	sta voice2Note

	jmp UpdateFrame
	
	


;**************** DATA ****************
;
; Pattern data = xxyyzzzz 
;		xx = arpeggio 0-3
;		yy = bass 0-3
;		zzzz = rootNote 1-F
; $00 = end of pattern

PatternTable:
	;dc.b $75, $75, 0
	dc.b $95, $86, $E7, $8B	; part 1
	dc.b $D5, $22, $BC, $BB	; part 2
	dc.b $29, $89, $29, $8E	; part 3
	dc.b $1E, $9C, $73, $73	; part 4
	dc.b $00

ArpeggioTable:
	dc.b %00111000		; 1 4 3	- maj7 3rd inversion
	dc.b %01100100		; 2 3 2 - 4 root
	dc.b %10111000		; 3 4 3 - min7 root
	dc.b %11010000		; min2 root
	
BassNote:	
	dc.b 21, 24, 26, 0
	
SIDsnapshot:
	dc.b  $70, $8F, $1F			; filter	cutoff_hi, res/filt, mode/vol
	
freq_hi = . + $100
freq_lo:
freqMult	dc.b $0C, $1C, $2D, $3F, $52, $66, $7B, $92, $AA, $C3, $DE, $FA


